'use strict';

const expect = require('chai').expect;
const update = require('../../methods/update');

describe('update', function() {

    let document;
    let schema;

    beforeEach(function() {
        document = {
            first_name: 'John',
            last_name: 'Doe',
            favorite_colors: ['blue', 'green']
        };

        schema = {
            fields: {
                first_name: { type: 'string', rule: { required: true } },
                last_name: { type: 'string', rule: { required: true } },
                favorite_colors: { type: 'array', rule: { required: false } }
            },
            key: [['last_name'], 'first_name', 'person_id']
        };
    });

    describe('removeCriticalFields', function(){

        it('should not alter the original document', function() {
            const first_name = document.first_name;
            const last_name = document.last_name;
            const favorite_colors = document.favorite_colors;

            update.removeCriticalFields(document, schema.key);

            expect(document).to.exist;
            expect(document.first_name).to.equal(first_name);
            expect(document.last_name).to.equal(last_name);
            expect(document.favorite_colors).to.equal(favorite_colors);
        });

        it('should remove critical fields from the specified document', function() {
            let result = update.removeCriticalFields(document, schema.key);

            expect(result).to.exist;
            expect(result.last_name).to.not.exist;
        });
    });

    describe('validation', function() {

        describe('hasRequiredFields', function() {

            it('should validate all model fields are present', function() {
                let badFn = function() {
                    delete document.favorite_colors;
    
                    update.hasRequiredFields(document, schema.fields);
                };
                
                expect(badFn).to.throw(Error, 'missing fields: favorite_colors');
            });
        });
    });
});